'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports['default'] = transition;
function addClass(elem, className) {
    if (elem.classList) {
        elem.classList.add(className);
    } else {
        elem.className += ' ' + className;
    }
    return elem;
}

function removeClass(elem, className) {
    if (elem.classList) {
        elem.classList.remove(className);
    } else {
        elem.className = elem.className.replace(new RegExp('(^|\\b)' + className.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
    }
    return elem;
}

function persistHistory(persistHistoryAs, history) {
    if (persistHistoryAs) {
        sessionStorage.setItem(persistHistoryAs, JSON.stringify(history));
    }
}

function createHistory() {
    var persistHistoryAs = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];

    var history = [];
    if (persistHistoryAs) {
        if (sessionStorage.getItem(persistHistoryAs)) {
            history = JSON.parse(sessionStorage.getItem(persistHistoryAs));
        }
    }
    return {
        persistHistoryAs: persistHistoryAs,
        history: history,
        push: function push(key) {
            this.history.push(key);
            persistHistory(this.persistHistoryAs, this.history);
        },
        pop: function pop() {
            this.history.pop();
            persistHistory(this.persistHistoryAs, this.history);
        },
        last: function last() {
            return this.history[this.history.length - 1] ? this.history[this.history.length - 1] : undefined;
        }
    };
}

function loadClasses(classList, lastElem, elem, direction) {
    addClass(elem.parentNode, classList.parent);
    addClass(lastElem, classList.lastElem);
    addClass(elem, classList.newElem);
    addClass(elem, classList.direction.replace('<direction>', direction));
}

function unloadClasses(classList, barrier, parentNode, elem, direction) {
    var newBarrier = barrier - 1;
    if (newBarrier > 0) {
        return newBarrier;
    }
    removeClass(elem, classList.newElem);
    removeClass(elem, classList.direction.replace('<direction>', direction));

    if (elem.parentNode) {
        removeClass(elem.parentNode, classList.parent);
    }
    return newBarrier;
}

function config(key, elem, isInit, ctx) {
    var _this = this;

    if (this.useHistory && !key) {
        throw new Error('Error in mithril-transition: ' + 'is required specified a key for the v-node.');
    }

    if (!isInit) {
        (function () {
            var parentNode = elem.parentNode;

            var direction = 'next';

            if (_this.useHistory) {
                /**
                 *  if the page that comes it was in the final
                 *  of the history queue i pop from that
                 */
                if (_this.history.last() === key) {
                    direction = 'prev';
                    _this.history.pop();
                } else {
                    /**
                     *  if direction is go the next i save the state
                     *  of the last element
                     */
                    if (_this.last) {
                        _this.history.push(_this.last.key);
                    }
                }
            }

            if (_this.last) {
                (function () {
                    var lastElem = _this.last.elem;
                    var id = 'mithril-transition-' + Date.now();
                    loadClasses(_this.classList, lastElem, elem, direction);
                    lastElem.setAttribute('data-transition-id', id);
                    parentNode.insertAdjacentHTML('beforeend', lastElem.outerHTML);
                    _this.last.elem = lastElem = parentNode.querySelector('[data-transition-id=' + id + ']');

                    var barrier = 2;
                    _this.anim(lastElem, elem, direction, function () {
                        lastElem.parentNode.removeChild(lastElem);
                        barrier = unloadClasses(_this.classList, barrier, parentNode, elem, direction);
                    }, function () {
                        barrier = unloadClasses(_this.classList, barrier, parentNode, elem, direction);
                    });
                })();
            }

            var userOnUnload = ctx.onunload;
            ctx.onunload = function () {
                elem.style['pointer-events'] = 'none';
                /**
                 *  the current element unloaded
                 *  is going to be the "new last element"
                 */
                _this.last = {
                    key: key,
                    elem: elem
                };

                if (userOnUnload) {
                    userOnUnload();
                }
            };
        })();
    }
}

function transition() {
    var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var anim = _ref.anim;
    var _ref$useHistory = _ref.useHistory;
    var useHistory = _ref$useHistory === undefined ? true : _ref$useHistory;
    var _ref$persistHistoryAs = _ref.persistHistoryAs;
    var persistHistoryAs = _ref$persistHistoryAs === undefined ? null : _ref$persistHistoryAs;
    var _ref$classList = _ref.classList;
    var classList = _ref$classList === undefined ? {
        parent: 'm-transition-parent',
        lastElem: 'm-transition-last-element',
        newElem: 'm-transition-new-element',
        direction: 'm-transition-<direction>'
    } : _ref$classList;

    if (!anim) {
        throw new Error('Error in mithril-transition: ' + 'option `anim` is required.');
    }

    var that = {
        useHistory: useHistory,
        anim: anim,
        config: config,
        classList: classList
    };

    if (that.useHistory) {
        that.history = createHistory(persistHistoryAs);
    }

    return function animate(elem, isInit, ctx) {
        that.config(this.attrs.key, elem, isInit, ctx);
    };
}

module.exports = exports['default'];
//# sourceMappingURL=index.js.map